import { JsSpawner } from '../../util';
import * as tempy from 'tempy';
import {
  DeviceCfgErrors,
  writeOrValidateDeviceCfgFile,
} from './write-or-validate-device-cfg-file';
import { runAndCatch } from '@carnesen/run-and-catch';
import { DEVICE_CONFIG_FILE_NAME } from '../../paths';
import { rimraf } from 'rimraf';

describe('Test writeOrValidateDeviceCfgFile', () => {
  let tmpDir;

  beforeAll(async () => {
    tmpDir = tempy.directory();
  });

  afterAll(async () => {
    rimraf(tmpDir);
  });

  test("file doesn't exist", async () => {
    const spawner = JsSpawner({ path: tmpDir });
    const deviceUuid = '1234';
    await writeOrValidateDeviceCfgFile({ spawner, deviceUuid });

    expect(spawner.exists(DEVICE_CONFIG_FILE_NAME)).toBeTruthy();
    expect(JSON.parse(await spawner.readFile(DEVICE_CONFIG_FILE_NAME))).toStrictEqual({
      systemId: 'production',
      deviceUuid,
    });
  });

  test('File exists but invalid json', async () => {
    const readFileOut = 'not valid json';
    const spawner = JsSpawner({ path: tmpDir });
    await spawner.writeFile(DEVICE_CONFIG_FILE_NAME, readFileOut);
    const deviceUuid = '1234';

    await writeOrValidateDeviceCfgFile({ spawner, deviceUuid });

    expect(spawner.exists(DEVICE_CONFIG_FILE_NAME)).toBeTruthy();
    expect(JSON.parse(await spawner.readFile(DEVICE_CONFIG_FILE_NAME))).toStrictEqual({
      systemId: 'production',
      deviceUuid,
    });
  });

  test('File exists but invalid contents', async () => {
    const readFileOut = JSON.stringify({
      notAField: 'not a field',
    });
    const spawner = JsSpawner({ path: tmpDir });
    await spawner.writeFile(DEVICE_CONFIG_FILE_NAME, readFileOut);
    const deviceUuid = '1234';

    await writeOrValidateDeviceCfgFile({ spawner, deviceUuid });

    expect(spawner.exists(DEVICE_CONFIG_FILE_NAME)).toBeTruthy();
    expect(JSON.parse(await spawner.readFile(DEVICE_CONFIG_FILE_NAME))).toStrictEqual({
      systemId: 'production',
      deviceUuid,
    });
  });

  test('File exists with mismatching device UUID', async () => {
    const readFileOut = JSON.stringify({
      systemId: 'production',
      deviceUuid: '5678',
    });
    const spawner = JsSpawner({ path: tmpDir });
    await spawner.writeFile(DEVICE_CONFIG_FILE_NAME, readFileOut);
    const deviceUuid = '1234';

    const exception = await runAndCatch(writeOrValidateDeviceCfgFile, {
      spawner,
      deviceUuid,
    });

    expect(exception.code).toBe(DeviceCfgErrors.TARGET_DEVICE_MISMATCH);
  });

  test('File exists with matching device UUID', async () => {
    const readFileOut = JSON.stringify({
      systemId: 'production',
      deviceUuid: '1234',
    });
    const spawner = JsSpawner({ path: tmpDir });
    await spawner.writeFile(DEVICE_CONFIG_FILE_NAME, readFileOut);
    const deviceUuid = '1234';

    await writeOrValidateDeviceCfgFile({ spawner, deviceUuid });

    expect(spawner.exists(DEVICE_CONFIG_FILE_NAME)).toBeTruthy();
    expect(JSON.parse(await spawner.readFile(DEVICE_CONFIG_FILE_NAME))).toStrictEqual({
      systemId: 'production',
      deviceUuid,
    });
  });
});
